unit Thumbs;

interface
uses Graphics,Jpeg,SysUtils;

type
  // Type qui represente la taille d'une image (largeur et heuteur)
  ImageTaille=record
    Width:integer;
    Height:integer;
  end;


procedure createThumb(srcFile,DestPath,prefix:String;size:integer;shadow,border:boolean;bsize:integer;bcolor:TColor);
function  getImageTaille(FilePath:String):imageTaille;

implementation

{=================================================================}
{ fonction renvoyant la taille d'une image                        }
{=================================================================}
function  getImageTaille(FilePath:String):imageTaille;
var Image:TPicture;
    returnValue:ImageTaille;
begin

  returnValue.Width:=0;
  returnvalue.Height:=0;
   Image:=TPicture.create;
  try

     Image.LoadFromFile(FilePath);
     returnValue.Width:=Image.Width;
     returnvalue.Height:=Image.Height;
  finally
   Image.Free;
  end;
  getImageTaille:= returnValue;
end;

{=================================================================}
{ procdure qui redimensionne une image de type JPG               }
{=================================================================}
procedure setSize(var myImage:TJPEGImage;sizeWidth,sizeHeight:integer);
var    myBmpImage:TBitMap;
begin

   myBmpImage:=TBitMap.create();
   try
     myBmpImage.Width:=sizeWidth;
     myBmpImage.Height:=sizeHeight;
     myBmpImage.PixelFormat :=pf32bit;
     myBmpImage.Canvas.StretchDraw(myBmpImage.Canvas.ClipRect,myImage);
     myImage.Assign(myBmpImage);
   finally
      myBmpImage.Free
   end;
end;


{=================================================================}
{ Fonction qui  retourne la taille proportionnelle de la hauteur  }
{ d'une largeur donne ou le contraire.                           }
{ la La distinction se fait par rapport au paramtre iswith       }
{ quand il est  true la fonction retourne la hauteur            }
{=================================================================}
function getProportion(Size:integer;taille:ImageTaille;isWith:boolean):integer;
begin
   if isWith then
     getProportion:=round(Size*taille.Width/taille.Height)
   else
     getProportion:=round(Size*taille.Height/taille.Width);
end;

{=================================================================}
{ Fonction qui convertie un entier  une couleur (TColor) .       }
{ Ex : la couleur R=192,G=192 et B=192  est clSilver (gris clair).}
{      IntToColor(192) reounrne  clSilver                         }
{ On peut reprendre la mme fonction pour convenir un code        }
{ RGB diffrent. On replacement  respectivement les C par         }
{ les valeurs RGB                                                 }
{ ((R and $FF) shl 16) + ((G and $FF) shl 8) + (B and $FF);       }
{=================================================================}
function IntToColor(C : byte): TColor;
begin
  Result := ((C and $FF) shl 16) + ((C and $FF) shl 8) + (C and $FF);
end;

{=================================================================}
{ Procdure qui permet de couper une image  une taille donne    }
{ pour ajustement la hauteur et la largeur.                       }
{=================================================================}
procedure Crop(var myImage:TJPEGImage;size:integer);
var    myBmpImage:TBitMap;

begin

   myBmpImage:=TBitMap.create();
   try
     myBmpImage.Width:=size;
     myBmpImage.Height:=size;
     myBmpImage.PixelFormat :=pf32bit;
     myBmpImage.Canvas.draw(((myImage.Width-size) div 2)*-1 ,((myImage.Height-size) div 2)*-1,myImage);
     myImage.Assign(myBmpImage);
   finally
      myBmpImage.Free
   end;
end;

{=================================================================}
{ Procdure qui permet de dessiner une bordure d'une taille et    }
{ d'une couleur donnes                                           }
{=================================================================}
procedure setBorder(var myImage:TJPEGImage;borderSize:integer;borderColor:TColor);
var myRectPos:integer;
    myBmpImage:TBitMap;
begin
   myBmpImage:=TBitMap.create();
   try
     if((borderSize mod 2)=0) then
       myRectPos:=(borderSize div 2)-1
     else myRectPos:=(borderSize div 2)-0;
     myBmpImage.Width:=myImage.Width+(borderSize *2);
     myBmpImage.Height:=myImage.Height+(borderSize * 2);
     myBmpImage.PixelFormat :=pf32bit;
     myBmpImage.Canvas.Pen.Color:=borderColor;
     myBmpImage.Canvas.Pen.Width:=borderSize;
     myBmpImage.Canvas.Rectangle(borderSize div 2,borderSize div 2,myBmpImage.Width-myRectPos,myBmpImage.Height-myRectPos);
     myBmpImage.Canvas.Draw(borderSize ,borderSize ,myImage);
     myImage.Assign(myBmpImage);
   finally
     myBmpImage.Free;
   end;
end;

{=================================================================}
{ Procdure qui agrandit la taille de la toile (Canvas)           }
{ de l'image, on spcifiant la couleur de l'arrire plan.         }
{=================================================================}
procedure SetCanvasSize(var myImage:TJPEGImage;size:integer;bcgrColor:TColor);
var    myBmpImage1,myBmpImage2:TBitMap;
       x,y:integer;
begin

   myBmpImage1:=TBitMap.create();
   myBmpImage2:=TBitMap.create();
   try
     myBmpImage1.Width:=myImage.Width;
     myBmpImage1.Height:=myImage.Height;
     myBmpImage2.Width:=myImage.Width+size;
     myBmpImage2.Height:=myImage.Height+size;
     myBmpImage1.PixelFormat :=pf24bit;
     myBmpImage2.PixelFormat :=pf24bit;
     myBmpImage2.Canvas.Pen.Color:=bcgrColor;
     myBmpImage2.Canvas.Brush.Color:=bcgrColor;
     myBmpImage2.Canvas.Rectangle(0,0,myBmpImage2.Width,myBmpImage2.Height);
     myBmpImage1.Canvas.Draw(0,0,myImage);
    
      for y:=0 to myBmpImage1.Width-1 do
     begin
        for x:=0 to myBmpImage1.Height-1 do
        begin
           myBmpImage2.Canvas.Pixels[x+(size div 2),y+(size div 2)]:=myBmpImage1.Canvas.Pixels[x,y];
        end;
     end;
     myImage.Assign(myBmpImage2);
   finally
      myBmpImage1.Free;
      myBmpImage2.Free;
   end;
end;

{=================================================================}
{ Procdure qui dessine un ombre de couleur noir, d'une distance  }
{ de 5 px  et de 120 degrs.                                     }
{ Dsole la personnalisation est pour bientt.                    }
{=================================================================}
procedure setShadow(var myImage:TJPEGImage;bcgrColor:TColor);
type  matrice=array[0..7,0..6] of integer;
const  coin:matrice=((19,41,69,105,135,157,172),(18,37,63,95,123,143,157),(16,32,54,82,105,123,134),(12,25,42,64,82,95,105),(8,16,28,42,54,63,69),(4,10,16,25,32,37,41),(2,4,8,12,16,18,19),(1,1,2,4,4,5,6));
       barre:array[0..7] of integer=(181,165,142,110,73,43,21,7);
var    myBmpImage:TBitMap;
       x,y:integer;
begin
   SetCanvasSize(myImage,20,bcgrColor);
   myBmpImage:=TBitMap.create();
   try
     myBmpImage.Width:=myImage.Width;
     myBmpImage.Height:=myImage.Height;
     myBmpImage.PixelFormat :=pf24bit;
     myBmpImage.Canvas.Draw(0,0,myImage);
     for y:=10 downto 4 do
       begin
        for x:=10 to 16 do
          begin
            //Coin Bas
            myBmpImage.Canvas.Pixels[x,myBmpImage.Height-y]:=IntToColor(255-coin[abs(y-10),x-10]);
            myBmpImage.Canvas.Pixels[myBmpImage.Width-(x-6),myBmpImage.Height-y]:=IntToColor(255-coin[abs(y-10),x-10]);
            //Coin Haut
             myBmpImage.Canvas.Pixels[myBmpImage.Height-y,x]:=IntToColor(255-coin[abs(y-10),x-10]);             
        end;
         //barre HorIsantal
         myBmpImage.Canvas.MoveTo(17,myBmpImage.Height-y);
         myBmpImage.Canvas.Pen.Color:=IntToColor(255-barre[abs(y-10)]);
         myBmpImage.Canvas.LineTo(myBmpImage.Height-10,myBmpImage.Width-y);
         //barre  Vertical
         myBmpImage.Canvas.MoveTo(myBmpImage.Height-y,17);
         myBmpImage.Canvas.Pen.Color:=IntToColor(255-barre[abs(y-10)]);
         myBmpImage.Canvas.LineTo(myBmpImage.Width-y,myBmpImage.Height-9);
       end;
     myImage.Assign(myBmpImage);
   finally
      myBmpImage.Free;
   end;
end;

{=================================================================}
{ Procdure qui cre une miniature d'une image.                   }
{                                                                 }
{=================================================================}
procedure createThumb(srcFile,DestPath,prefix:String;size:integer;shadow,border:boolean;bsize:integer;bcolor:TColor);
var taille:ImageTaille;
    filename,filepath:String;
    myImageSrc,myImageDes:TJPEGImage;
begin
  filename:=ExtractFilename(srcFile);
  filepath:=ExtractFilePath(srcFile);
  taille:=getImageTaille(srcFile);

  myImageSrc:=TJPEGImage.Create();
  myImageDes:=TJPEGImage.Create();
  try
     myImageSrc.PixelFormat:=jf24Bit;
     myImageDes.PixelFormat:=jf24Bit;
     myImageSrc.CompressionQuality:=100;
     myImageDes.CompressionQuality:=100;
     myImageSrc.LoadFromFile(srcFile);
     
     if taille.Width > taille.Height then
       setSize(myImageSrc,getProportion(size,taille,true),size)
     else setSize(myImageSrc,size,getProportion(size,taille,false));
     crop(myImageSrc,size);
     if border then
        setBorder(myImageSrc,bsize,bcolor);     
     if shadow then
        setShadow(myImageSrc,clWhite);

     myImageDes.Assign(myImageSrc);

     myImageDes.SaveToFile(DestPath+'\'+prefix+filename);
   finally
     myImageSrc.Free;
     myImageDes.Free;
   end;
     
end;

end.
